<?php

function contextly_overlay_editor_page($type) {
  print ContextlyDrupalKit::getInstance()
    ->newOverlayDialog($type)
    ->render();
}

function contextly_ajax_editor_request_callback($request, $node) {
  try {
    // Build parameters list.
    $params = $_POST;
    $params['custom_id'] = $node->nid;

    // Handle the request.
    $result = ContextlyDrupalKit::getInstance()
      ->newWidgetsEditor()
      ->handleRequest($request, $params);

    // Deliver results to the client.
    _contextly_json_output($result);
  }
  catch (ContextlyKitException $e) {
    _contextly_watchdog_exception($e);
    _contextly_return_internal_error();
  }
}

function contextly_ajax_put_node_callback($node) {
  try {
    // Node update is allowed to everyone, so we use per-node lock to avoid high
    // server and Contextly load by parallel node views at the same time.
    $lock_name = "contextly-put-node:{$node->nid}";
    if (lock_acquire($lock_name)) {
      ContextlyDrupalKit::getInstance()
        ->newDrupalNodeEditor()
        ->putNode($node);
      lock_release($lock_name);
    }

    // Notify client about success, even on lock acquired by another request.
    _contextly_json_output(array(
      'success' => TRUE,
    ));
  }
  catch (ContextlyKitException $e) {
    _contextly_watchdog_exception($e);
    _contextly_return_internal_error();
  }
}

function _contextly_return_internal_error() {
  drupal_set_header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error');
  exit;
}

/**
 * Outputs data in JSON format.
 *
 * Port of drupal_json_output() from D7.
 *
 * @param $var
 *   (optional) If set, the variable will be converted to JSON and output.
 */
function _contextly_json_output($var = NULL) {
  // We are returning JSON, so tell the browser.
  drupal_set_header('Content-Type: application/json');

  if (isset($var)) {
    echo _contextly_json_encode($var);
  }
}
