(function ($) {

  Drupal.contextly = Drupal.contextly || {};

  Drupal.contextly.snippetEdit = {
    addContainers: function(containers) {
      containers.show();

      // Apply buttons state depending on editor state.
      if (Contextly.editor.isLoaded) {
        this.onSettingsLoaded();
      }
      else if (Contextly.editor.hasFailed) {
        this.onSettingsFailed();
      }
      else if (Contextly.editor.isLoading) {
        this.onSettingsLoading();
      }
    },

    getContainers: function() {
      return $('.d-ctx-snippet-edit');
    },

    getButtons: function() {
      return this.getContainers().find('.d-ctx-snippet-button');
    },

    getDescriptions: function() {
      return this.getContainers().find('.d-ctx-snippet-status');
    },

    getButtonLabel: function() {
      var hasLinks = Drupal.settings.contextlyEditor
        && Drupal.settings.contextlyEditor.snippet
        && Drupal.settings.contextlyEditor.snippet.links;
      if (hasLinks) {
        return Drupal.t('Edit Related Posts');
      }
      else {
        return Drupal.t('Choose Related Posts');
      }
    },

    onSettingsLoaded: function () {
      this.getButtons()
        .attr('value', this.getButtonLabel())
        .unbind('click.contextly')
        .bind('click.contextly', function (e) {
          e.preventDefault();
          Contextly.editor.open('snippet');
        });
      this.getDescriptions()
        .empty()
        .hide();
    },

    onSettingsFailed: function () {
      this.getButtons()
        .attr('value', Drupal.t('Try again'))
        .unbind('click.contextly')
        .bind('click.contextly', function (e) {
          e.preventDefault();
          Contextly.editor.loadSettings();
        });
      this.getDescriptions()
        .empty()
        .text(Drupal.t('Loading of the Contextly settings has failed. Click "Try again" button to make another attempt.'))
        .show();
    },

    onSettingsLoading: function () {
      this.getButtons()
        .attr('value', Drupal.t('Loading...'))
        .unbind('click.contextly')
        .bind('click.contextly', function (e) {
          e.preventDefault();
        });
      this.getDescriptions()
        .empty()
        .hide();
    },

    onWidgetUpdated: function() {
      this.getButtons()
        .attr('value', this.getButtonLabel());
    },

    attach: function (context, settings) {
      if (!Contextly || !Contextly.editor) {
        return;
      }

      $('body', context).once('d-ctx-snippet-edit', this.proxy(function() {
        $(window)
          .bind('contextlySettingsLoading', this.proxy(this.onSettingsLoading))
          .bind('contextlySettingsLoaded', this.proxy(this.onSettingsLoaded))
          .bind('contextlySettingsFailed', this.proxy(this.onSettingsFailed))
          .bind('contextlyWidgetUpdated', this.proxy(this.onWidgetUpdated))
          .bind('contextlyWidgetRemoved', this.proxy(this.onWidgetUpdated));
      }));

      var containers = $('.d-ctx-snippet-edit', context)
        .once('d-ctx-snippet-edit');
      if (containers.length) {
        this.addContainers(containers);
      }
    },

    proxy: function(callback) {
      var self = this;
      return function() {
        return callback.apply(self, arguments);
      }
    }
  };

  Drupal.behaviors.contextlyNodeFormSnippetEdit = function (context) {
    var se = Drupal.contextly.snippetEdit;

    // React to the window events. Attach only once for the page.
    var markerClass = 'd-ctx-snippet-edit-processed';
    $('body:not(.' + markerClass + ')', context)
      .addClass(markerClass)
      .each(function() {
        $(window)
          .bind('contextlySettingsLoading', se.proxy(se.onSettingsLoading))
          .bind('contextlySettingsLoaded', se.proxy(se.onSettingsLoaded))
          .bind('contextlySettingsFailed', se.proxy(se.onSettingsFailed))
          .bind('contextlyWidgetUpdated', se.proxy(se.onWidgetUpdated))
          .bind('contextlyWidgetRemoved', se.proxy(se.onWidgetUpdated));
      });

    markerClass = 'd-ctx-snippet-edit-processed';
    var containers = $('.d-ctx-snippet-edit:not(.' + markerClass + ')', context)
      .addClass(markerClass);
    if (containers.length) {
      se.addContainers(containers);
    }
  };

})(jQuery);
