<?php

/**
 * Returns HTML for the Contextly widget wrapper.
 *
 * @param object|null $node
 */
function theme_contextly_widget($node) {
  if (empty($node->nid)) {
    return '';
  }

  // Export settings.
  $settings = _contextly_node_widgets_settings($node);
  _contextly_add_js_setting_once('widget', $settings);

  // Include JS and report to user if it fails.
  $library_included = _contextly_add_library('node-view');
  if (!$library_included && user_access('administer contextly')) {
    $error_message = t('The Contextly widget is not able to load because of missing libraries. See log for details.');
    drupal_set_message($error_message, 'status', FALSE);
  }

  // Build the widget wrapper.
  $attributes = array(
    'id' => 'ctx-module',
    'class' => 'ctx-module-container ctx-clearfix',
  );
  $output = '<div' . drupal_attributes($attributes) . '></div>';

  return $output;
}

function theme_contextly_metadata($metadata) {
  $attributes = array(
    'name' => 'contextly-page',
    'id' => 'contextly-page',
  );

  // All special characters are encoded inside JSON (<, >, ', &, and "). Double
  // quotes around JSON property names and values are fine here, because we use
  // single quotes around HTML attribute value.
  $metadata = _contextly_json_encode($metadata);
  return '<meta' . drupal_attributes($attributes) . " content='" . $metadata . "' />\n";
}

function theme_contextly_snippet_edit() {
  $output = '<input type="button" class="d-ctx-snippet-button form-submit" value="" />' . "\n";
  $output .= '<div class="d-ctx-snippet-status description"></div>' . "\n";
  $output = '<div class="d-ctx-snippet-edit" style="display: none;">' . $output . "</div>\n";
  return $output;
}
